/*
 * Copyright 2007-2017 Charles du Jeu - Abstrium SAS <team (at) pyd.io>
 * This file is part of Pydio.
 *
 * Pydio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pydio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Pydio.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The latest code can be found at <https://pydio.com>.
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _materialUi = require('material-ui');

var React = require('react');
exports['default'] = React.createClass({
    displayName: 'ServicesList',

    mixins: [AdminComponents.MessagesConsumerMixin],

    propTypes: {
        dataModel: React.PropTypes.instanceOf(PydioDataModel).isRequired,
        rootNode: React.PropTypes.instanceOf(AjxpNode).isRequired,
        currentNode: React.PropTypes.instanceOf(AjxpNode).isRequired,
        filter: React.PropTypes.string,
        details: React.PropTypes.bool
    },

    reload: function reload() {
        this.props.currentNode.reload();
    },

    /**
     * @param node PydioNode
     * @returns {XML}
     */
    renderListItem: function renderListItem(node) {
        var description = undefined;
        if (this.props.details) {
            var nodes = '';
            var metaNode = node.getMetadata().get('nodes');
            if (metaNode) {
                metaNode = JSON.parse(metaNode);
                if (metaNode && metaNode.length) {
                    nodes = ' - Running on ' + metaNode.join(',');
                }
            }
            description = node.getMetadata().get('description') + nodes;
        }
        return React.createElement(_materialUi.ListItem, {
            primaryText: node.getLabel(),
            secondaryText: description,
            leftCheckbox: React.createElement(_materialUi.Checkbox, { defaultChecked: node.getMetadata().get("status") === 'STARTED', checked: node.getMetadata().get("status") === 'STARTED' })
        });
    },

    filterNodes: function filterNodes(node) {
        if (!this.props.filter) {
            return true;
        }
        return node.getMetadata().get('status') === this.props.filter;
    },

    render: function render() {
        var _this = this;

        var currentNode = this.props.currentNode;

        var services = currentNode.getChildren();
        var tags = new Map();
        services.forEach(function (service) {
            if (!_this.filterNodes(service)) {
                return;
            }
            var tag = service.getMetadata().get('tag');
            if (!tag) {
                tag = 'General';
            }
            if (!tags.has(tag)) {
                tags.set(tag, []);
            }
            tags.get(tag).push(service);
        });
        var listItems = [];
        tags.forEach(function (items, tag) {
            listItems.push(React.createElement(
                _materialUi.Subheader,
                { inset: false },
                tag.charAt(0).toUpperCase() + tag.substr(1)
            ));
            for (var i = 0; i < items.length; i++) {
                listItems.push(_this.renderListItem(items[i]));
            }
            listItems.push(React.createElement(_materialUi.Divider, { inset: false }));
        });
        listItems.pop(); // remove last divider

        return React.createElement(
            _materialUi.List,
            { style: { backgroundColor: 'white' }, className: "layout-fill" },
            listItems
        );
    }

});
module.exports = exports['default'];
