/*
 * Copyright 2007-2017 Charles du Jeu - Abstrium SAS <team (at) pyd.io>
 * This file is part of Pydio.
 *
 * Pydio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pydio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Pydio.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The latest code can be found at <https://pydio.com>.
 */
const React = require('react');
import {List, ListItem, Subheader, Divider, Checkbox, FontIcon} from 'material-ui'

export default React.createClass({

    mixins:[AdminComponents.MessagesConsumerMixin],

    propTypes:{
        dataModel:React.PropTypes.instanceOf(PydioDataModel).isRequired,
        rootNode:React.PropTypes.instanceOf(AjxpNode).isRequired,
        currentNode:React.PropTypes.instanceOf(AjxpNode).isRequired,
        filter:React.PropTypes.string,
        details: React.PropTypes.bool
    },

    reload:function(){
        this.props.currentNode.reload();
    },

    /**
     * @param node PydioNode
     * @returns {XML}
     */
    renderListItem: function(node){
        let description;
        if(this.props.details) {
            let nodes = '';
            let metaNode = node.getMetadata().get('nodes');
            if(metaNode) {
                metaNode = JSON.parse(metaNode);
                if(metaNode && metaNode.length){
                    nodes = ' - Running on ' + metaNode.join(',')
                }
            }
            description = node.getMetadata().get('description') + nodes
        }
        return (
            <ListItem
                primaryText={node.getLabel()}
                secondaryText={description}
                leftCheckbox={<Checkbox defaultChecked={node.getMetadata().get("status") === 'STARTED'} checked={node.getMetadata().get("status") === 'STARTED'}/>}
            />);
    },


    filterNodes:function(node){
        if(! this.props.filter ) {
            return true;
        }
        return node.getMetadata().get('status') === this.props.filter;
    },

    render:function(){
        const {currentNode} = this.props;
        const services = currentNode.getChildren();
        let tags = new Map();
        services.forEach((service) => {
            if (!this.filterNodes(service)){
                return;
            }
            let tag = service.getMetadata().get('tag');
            if(!tag) {
                tag = 'General';
            }
            if(!tags.has(tag)) {
                tags.set(tag, []);
            }
            tags.get(tag).push(service);
        });
        let listItems = [];
        tags.forEach((items, tag) => {
            listItems.push(<Subheader inset={false}>{tag.charAt(0).toUpperCase() + tag.substr(1)}</Subheader>);
            for(let i=0; i < items.length; i++){
                listItems.push(this.renderListItem(items[i]));
            }
            listItems.push(<Divider inset={false}/>);
        });
        listItems.pop(); // remove last divider

        return (
            <List style={{backgroundColor: 'white'}} className={"layout-fill"}>
                {listItems}
            </List>
        );
    }

});
