/*
 * Copyright 2007-2017 Charles du Jeu - Abstrium SAS <team (at) pyd.io>
 * This file is part of Pydio.
 *
 * Pydio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pydio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Pydio.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The latest code can be found at <https://pydio.com>.
 */
import React from 'react';
import {Divider, List} from 'material-ui';

const Pydio = require('pydio');
const { PydioContextConsumer, moment } = Pydio.requireLib('boot');
import AS2Client from './Client'
import Activity from './Activity'

class ActivityList extends React.Component {

    constructor(props) {
        super(props);
        if(props.items){
            this.state = {data:{items:props.items}};
        } else {
            this.state = {data:[]};
        }
    }

    componentWillMount(){
        if(this.props.items) {
            return
        }
        if(this.props.contextData) {
            AS2Client.loadActivityStreams((json) => {
                this.setState({data: json});
            }, this.props.context, this.props.contextData, 'outbox', this.props.pointOfView)
        }
    }

    componentWillReceiveProps(nextProps) {
        if(nextProps.items) {
            this.setState({data:{items:nextProps.items}});
            return;
        }
        if (nextProps.contextData !== this.props.contextData || nextProps.context != this.props.context) {
            AS2Client.loadActivityStreams((json) => {
                this.setState({data: json});
            }, nextProps.context, nextProps.contextData, 'outbox', nextProps.pointOfView)
        }
    }

    render(){

        let content = [];
        let {data} = this.state;
        const {listContext, groupByDate} = this.props;
        let previousFrom;
        if (data !== null && data.items) {
            data.items.forEach(function(ac){

                let fromNow = moment(ac.updated).fromNow();
                if (groupByDate && fromNow !== previousFrom) {
                    content.push(<div style={{padding: '0 16px', fontSize: 13, color: 'rgba(147, 168, 178, 0.67)', fontWeight: 500}}>{fromNow}</div>);
                }
                content.push(<Activity key={ac.id} activity={ac} listContext={listContext} oneLiner={groupByDate}/>);
                if (groupByDate) {
                    previousFrom = fromNow;
                }

            });
        }
        return (
            <List style={this.props.style}>{content}</List>
        );
    }

}

ActivityList.PropTypes = {
    context: React.PropTypes.string,
    contextData: React.PropTypes.string,
    boxName : React.PropTypes.string,
    pointOfView: React.PropTypes.oneOf(['GENERIC', 'ACTOR', 'SUBJECT'])
};

ActivityList = PydioContextConsumer(ActivityList);
export {ActivityList as default}