/*
 * Copyright 2007-2017 Charles du Jeu - Abstrium SAS <team (at) pyd.io>
 * This file is part of Pydio.
 *
 * Pydio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pydio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Pydio.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The latest code can be found at <https://pydio.com>.
 */

'use strict';

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var React = require('react');

var _require$requireLib = require('pydio').requireLib('boot');

var PydioContextConsumer = _require$requireLib.PydioContextConsumer;

var ChatClient = require('./ChatClient');
var Message = require('./Message');

var Chat = (function (_React$Component) {
    _inherits(Chat, _React$Component);

    function Chat(props) {
        _classCallCheck(this, Chat);

        _React$Component.call(this, props);
        this.client = null;
        this.state = { messages: [], room: null, value: "" };
    }

    Chat.prototype.componentDidMount = function componentDidMount() {
        this.start();
    };

    Chat.prototype.componentWillUnmount = function componentWillUnmount() {
        this.stop();
    };

    Chat.prototype.componentDidUpdate = function componentDidUpdate() {
        this.refs.comments.scrollTop = 100000;
    };

    Chat.prototype.onNewMessage = function onNewMessage(msg) {
        if (msg) {
            var data = JSON.parse(msg['data']);
            if (data['@type'] === 'ROOM_UPDATE') {
                this.setState({ room: data['Room'] });
            }
            this.setState({ messages: this.client.getMessages() });
        }
    };

    Chat.prototype.start = function start() {
        var _props = this.props;
        var pydio = _props.pydio;
        var roomType = _props.roomType;
        var roomObjectId = _props.roomObjectId;

        this.client = new ChatClient(pydio, roomType, roomObjectId, this.onNewMessage.bind(this));
        this.client.open();
    };

    Chat.prototype.stop = function stop() {
        this.client.close();
    };

    Chat.prototype.postMessage = function postMessage() {
        if (!this.state.value) {
            return;
        }
        if (!this.state.room || !this.state.room.Uuid) {
            return;
        }
        var message = {
            "@type": "POST",
            "Message": { "RoomUuid": this.state.room.Uuid, "Message": this.state.value }
        };
        this.client.send(JSON.stringify(message));
        this.setState({ value: '' });
    };

    Chat.prototype.keyDown = function keyDown(event) {
        if (event.key === 'Enter') {
            event.preventDefault();
            this.postMessage();
        }
    };

    Chat.prototype.render = function render() {
        var _this = this;

        var messages = this.state.messages;

        var data = [];
        messages.forEach(function (m) {
            var mData = JSON.parse(m['data']);
            if (mData['Message']) {
                data.push(React.createElement(Message, { key: mData.Uuid, message: mData }));
            }
        });
        return React.createElement(
            'div',
            { style: { padding: 0 } },
            React.createElement(
                'div',
                { ref: 'comments', className: 'comments_feed', style: { maxHeight: 300, overflowY: 'auto' } },
                data
            ),
            React.createElement(MaterialUI.Divider, null),
            React.createElement(
                'div',
                { style: { backgroundColor: 'white' } },
                React.createElement(MaterialUI.TextField, {
                    hintText: this.props.pydio.MessageHash['meta.comments.2'],
                    hintStyle: { whiteSpace: 'nowrap' },
                    value: this.state.value,
                    onChange: function (event, newValue) {
                        _this.setState({ value: newValue });
                    },
                    multiLine: true,
                    ref: 'new_comment',
                    onKeyDown: this.keyDown.bind(this),
                    fullWidth: true,
                    underlineShow: false,
                    style: { marginLeft: 16, marginRight: 16 }
                })
            )
        );
    };

    return Chat;
})(React.Component);

Chat.PropTypes = {
    roomType: React.PropTypes.string,
    roomObjectId: React.PropTypes.string
};

exports['default'] = Chat = PydioContextConsumer(Chat);
exports['default'] = Chat;
module.exports = exports['default'];
