/*
 * Copyright 2007-2017 Charles du Jeu - Abstrium SAS <team (at) pyd.io>
 * This file is part of Pydio.
 *
 * Pydio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pydio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Pydio.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The latest code can be found at <https://pydio.com>.
 */

const PydioWebSocket = require('pydio/http/websocket');
const ReconnectingWebSocket = require('reconnecting-websocket');

class ChatClient extends PydioWebSocket{

    constructor(pydioObject, roomType, objectId, messageHandler) {
        super(pydioObject);
        this.currentRepo = pydioObject.repositoryId;
        this.messageHandler = messageHandler;
        this.messages = [];
        this.roomType = roomType;
        this.objectId = objectId;
    }

    send(message){
        this.ws.send(message);
    }

    open(){
        this.close();
        if (!this.currentRepo) {
            return;
        }
        const url = this.pydio.getPluginConfigs("core.conf").get("ENDPOINT_WEBSOCKET").replace("/ws", "/chat");
        this.ws = new ReconnectingWebSocket(url);

        this.ws.addEventListener('open', () => {
            PydioWebSocket.subscribeJWT(this.ws);
            this.joinRoom(this.ws);
        });
        this.ws.addEventListener('message', (msg) => {
            console.log(msg);
            this.messages.push(msg);
            this.messageHandler(msg);
        });
        this.ws.addEventListener('close', (event) => {
            PydioWebSocket.logClose(event)
        });
    }

    close(){
        if(this.ws === null) {
            return;
        }
        this.leaveRoom(this.ws);
        super.close();
    }

    joinRoom(ws){
        let message = {"@type" : "JOIN", "Room":{"Type":this.roomType, "RoomTypeObject":this.objectId}};
        ws.send(JSON.stringify(message));
    }

    leaveRoom(ws){
        let message = {"@type" : "LEAVE", "Room":{"Type":this.roomType, "RoomTypeObject":this.objectId}};
        ws.send(JSON.stringify(message));
        this.messages = [];
        this.messageHandler();
    }

    getMessages() {
        return this.messages;
    }

}

export {ChatClient as default};